
/**
 * A demonstration application to be used as a start for homework #1
 * for CMSC	427	- Spring 2000.
 * 
 * @author Ben Bederson

 * @date January 25, 2000
 * 
 * Modified	and	turned in by Justin	Gaertner
 * login:  bb42708
 * 
 * Turned in as part of Homework 2, Feb. 15, 2000.
 * 
 */
import java.awt.*;
import java.awt.event.*;

/**
 * This	is the drawing panel that we use to	draw on. We	must create	this class
 * extending Panel so that we can override the crucial 'paint' method which
 * gets	called whenever	the	system determines that Panel needs to be redrawn.
 * We also define simple event handlers	to determine when the user clicks
 * on the drawing panel.
 */
class CircleDrawingPanel extends Panel
{
	private boolean antialias;
	private int x1, y1, x2, y2; // Center and point on circle
	static int[] grayArray = new int[101];
	/**
	 * Simple constructor that adds	a mouse	listener to	this panel.
	 */
	
	public CircleDrawingPanel()	{
		MouseListener mouseListener	= new DrawingMouseListener(this);
		addMouseListener(mouseListener);
		
		antialias = false;
		double grayRange = 255.0/100;
		grayArray[0] = 255;
		grayArray[100] = 0;
		//  Put the values into the color array grayArray...
		for	(int i = 1;	i<100;	i++) {
			grayArray[i] = (int) (grayRange*(100-i));
		}
	}
	
	/**
	 * This	method gets	called whenever	the	panel needs	to get repainted. 
	 * Putting in drawing methods here,	such as	g.drawLine() will cause
	 * those things	to be drawn	on the panel. For instance,	if you add
	 * g.drawLine(0, 0,	100, 100) the canvas will appear with a	diagonal line.
	 */
	public void	paint(Graphics g) {
		g.drawLine(x1,y1,x1,y1);
		g.drawLine(x2,y2,x2,y2);
		
		double radius =	Math.sqrt(Math.pow(x2-x1,2)	+ Math.pow(y2-y1,2));
		int	x =	0;
		int	y =	(int) radius;
		int	e =	3 -	2 *	y;
		/**
		* I got the code for e (and how it changes at the
		* end of the loop, farther down in the code) 
		* on the Internet.  It is the traditional Bresenham
		* algorithm.
		*/
		
		int	posx, negx,	posy, negy;
		double operand, ylower, yupper, theta;
		int percent, pixelupper, pixellower, colorvalue;
		boolean	flag = true;
		boolean xequalsy = false;
		while (flag) {
			if (!antialias) {
				/**
				* This is the code for the non-antialiased circle.
				*/
				
				/**
				* We calculate what to draw using the 
				* second octant (45 degrees to 90 degrees),
				* and then reflect it into the other octants.
				* The same thing happens in the code for
				* the antialiased circles.
				*/
				posx = x1+x;
				negx = x1-x;
				posy = y1+y;
				negy = y1-y;
				
				g.drawLine(posx,posy,posx,posy);
				g.drawLine(posx,negy,posx,negy);
				g.drawLine(negx,posy,negx,posy);
				g.drawLine(negx,negy,negx,negy);
				
				posx = x1+y;
				negx = x1-y;
				posy = y1+x;
				negy = y1-x;
				
				g.drawLine(posx,posy,posx,posy);
				g.drawLine(posx,negy,posx,negy);
				g.drawLine(negx,posy,negx,posy);
				g.drawLine(negx,negy,negx,negy); 
			}
			
			else {
				/**
				* This is the code for antialiasing the circle.
				*/
				
				operand = Math.pow(radius,2) - Math.pow((x+.5),2);
				if (operand >=0 ) {
					ylower = Math.sqrt(operand);
				} else {
					break;
				}
				theta = Math.asin((x)/radius);
				yupper = ylower + Math.cos(theta) + Math.cos(Math.PI/2-theta);
				
				for (int count = -1; count<3; count++) {
					pixellower = y+count;
					pixelupper = pixellower + 1;
					
					if (pixellower > yupper || pixelupper < ylower) {
						colorvalue = 0;
					} else if (pixellower < yupper && pixelupper > yupper) {
						colorvalue = (int) (100 * (yupper - pixellower));
					} else if (pixelupper > ylower && pixellower < ylower) {
						colorvalue = (int) (100 * (pixelupper - ylower));
					} else {
						colorvalue = 100;
					};
					 
					 if (colorvalue < 0) break;
					 
					 g.setColor(new Color(grayArray[colorvalue],grayArray[colorvalue],grayArray[colorvalue]));
					 
					 posx = x1+x;
					 negx = x1-x;
					 posy = y1+y+count;
					 negy = y1-y-count;
					 
					 g.drawLine(posx,posy,posx,posy);
					 g.drawLine(posx,negy,posx,negy);
					 g.drawLine(negx,posy,negx,posy);
					 g.drawLine(negx,negy,negx,negy);
					 
					 posx = x1+y+count;
					 negx = x1-y-count;
					 posy = y1+x;
					 negy = y1-x;
					 
					 g.drawLine(posx,posy,posx,posy);
					 g.drawLine(posx,negy,posx,negy);
					 g.drawLine(negx,posy,negx,posy);
					 g.drawLine(negx,negy,negx,negy);
				}
			}
			
			/**
			* The rest of the loop changes e by the 
			* appropriate amount and then determines if
			* the loop is over.  This part of the code 
			* applies to both antialiased and non-antialiased
			* circles.
			*/
			
			if (e <	0) {
				e =	e +	4 *	x +	6;
			} else {
				e =	e +	4 *	(x - y)	+ 10;
				y--;
			}
			
			x++;
			
			/**
			* The boolean variable xequalsy keeps track
			* of if x and y have shared the same value.
			* It is important that at some point they share 
			* the same value, or the antialiased circle may
			* have gaps at 45 degrees (and reflections).
			* 
			* The boolean variable flag determines if the
			* main loop should be continued.
			*/
			
			if (x==y) {
				xequalsy = true;;
			} else if (x>y) {
				if (xequalsy) {
					flag = false;
				} else {
					y=x;
					xequalsy = true;
				}
			}
		}
	}

	/**
	 * Specify the center of the circle	to be drawn	in this	panel.
	 */
	public void	setPoint1(int x, int y)	{
		x1 = x;
		y1 = y;
		repaint();
	}

	/**
	 * Specify a point on the circle to	be drawn in	this panel.
	 */
	public void	setPoint2(int x, int y)	{
		x2 = x;
		y2 = y;
		repaint();
	}	
	
	/**
	 * Changes whether we draw use the regular algorithm
	 * or the antialiasing algorithm to draw the circle,
	 * and then redraws the circle.
	 */
	
	public void changeAntialias() {
		antialias = !antialias;
		repaint();
	}
}

